#!/usr/bin/env bash
###
 # Tests for the Arg/Kwarg handling section of the utils script.
 ##


@test "Arg/Kwarg Handling - Empty" {
    # Import utils script.
    source ../utils.sh

    handle_args_kwargs
    [[ "${#args[@]}" == 0 ]]
    [[ "${#kwargs[@]}" == 0 ]]
    [[ "${#flags[@]}" == 0 ]]
    [[ ${help_flags} == false ]]
}


@test "Arg/Kwarg Handling - Help Flags Set" {
    # Import utils script.
    source ../utils.sh

    # Test -h flag.
    handle_args_kwargs -h
    [[ "${#args[@]}" == 0 ]]
    [[ "${#kwargs[@]}" == 0 ]]
    [[ "${#flags[@]}" == 0 ]]
    [[ ${help_flags} == true ]]

    # Reset flag.
    help_flags=false
    [[ ${help_flags} == false ]]

    # Test --help flag.
    handle_args_kwargs --help
    [[ "${#args[@]}" == 0 ]]
    [[ "${#kwargs[@]}" == 0 ]]
    [[ "${#flags[@]}" == 0 ]]
    [[ ${help_flags} == true ]]
}


@test "Arg/Kwarg Handling - Args Only" {
    # Import utils script.
    source ../utils.sh

    # Test with one arg.
    expected_result=("one")
    handle_args_kwargs one
    [[ "${#args[@]}" == 1 ]]
    [[ ${args[@]} == ${expected_result[@]} ]]
    [[ "${#kwargs[@]}" == 0 ]]
    [[ "${#flags[@]}" == 0 ]]
    [[ ${help_flags} == false ]]

    # Ensure args clear on new call.
    handle_args_kwargs
    [[ "${#args[@]}" == 0 ]]

    # Test with two args.
    expected_result=("one" "two")
    handle_args_kwargs one two
    [[ "${#args[@]}" == 2 ]]
    [[ ${args[@]} == ${expected_result[@]} ]]
    [[ "${#kwargs[@]}" == 0 ]]
    [[ "${#flags[@]}" == 0 ]]
    [[ ${help_flags} == false ]]

    # Ensure args clear on new call.
    handle_args_kwargs
    [[ "${#args[@]}" == 0 ]]

    # Test with three args.
    expected_result=("one" "two" "three")
    handle_args_kwargs one two three
    [[ "${#args[@]}" == 3 ]]
    [[ ${args[@]} == ${expected_result[@]} ]]
    [[ "${#kwargs[@]}" == 0 ]]
    [[ "${#flags[@]}" == 0 ]]
    [[ ${help_flags} == false ]]
}


@test "Arg/Kwarg Handling - Kwargs Only - V1" {
    # Import utils script.
    source ../utils.sh

    declare -A expected_result=()

    # Test with one kwargs.
    expected_result[a]=x
    handle_args_kwargs -a x
    [[ "${#args[@]}" == 0 ]]
    [[ "${#kwargs[@]}" == 1 ]]
    [[ ${kwargs[@]} == ${expected_result[@]} ]]
    [[ "${#flags[@]}" == 0 ]]
    [[ ${help_flags} == false ]]

    # Ensure kwargs clear on new call.
    handle_args_kwargs
    [[ "${#kwargs[@]}" == 0 ]]

    # Test with two kwargs.
    expected_result[b]=y
    handle_args_kwargs -a x -b y
    [[ "${#args[@]}" == 0 ]]
    [[ "${#kwargs[@]}" == 2 ]]
    [[ ${kwargs[@]} == ${expected_result[@]} ]]
    [[ "${#flags[@]}" == 0 ]]
    [[ ${help_flags} == false ]]

    # Ensure kwargs clear on new call.
    handle_args_kwargs
    [[ "${#kwargs[@]}" == 0 ]]

    # Test with three kwargs.
    expected_result[c]=z
    handle_args_kwargs -a x -b y -c z
    [[ "${#args[@]}" == 0 ]]
    [[ "${#kwargs[@]}" == 3 ]]
    [[ ${kwargs[@]} == ${expected_result[@]} ]]
    [[ "${#flags[@]}" == 0 ]]
    [[ ${help_flags} == false ]]
}


@test "Arg/Kwarg Handling - Kwargs Only - V2" {
    # Import utils script.
    source ../utils.sh

    declare -A expected_result=()

    # Test with one kwargs.
    expected_result[a]=x
    handle_args_kwargs --a x
    [[ "${#args[@]}" == 0 ]]
    [[ "${#kwargs[@]}" == 1 ]]
    [[ ${kwargs[@]} == ${expected_result[@]} ]]
    [[ "${#flags[@]}" == 0 ]]
    [[ ${help_flags} == false ]]

    # Ensure kwargs clear on new call.
    handle_args_kwargs
    [[ "${#kwargs[@]}" == 0 ]]

    # Test with two kwargs.
    expected_result[b]=y
    handle_args_kwargs --a x --b y
    [[ "${#args[@]}" == 0 ]]
    [[ "${#kwargs[@]}" == 2 ]]
    [[ ${kwargs[@]} == ${expected_result[@]} ]]
    [[ "${#flags[@]}" == 0 ]]
    [[ ${help_flags} == false ]]

    # Ensure kwargs clear on new call.
    handle_args_kwargs
    [[ "${#kwargs[@]}" == 0 ]]

    # Test with three kwargs.
    expected_result[c]=z
    handle_args_kwargs --a x --b y --c z
    [[ "${#args[@]}" == 0 ]]
    [[ "${#kwargs[@]}" == 3 ]]
    [[ ${kwargs[@]} == ${expected_result[@]} ]]
    [[ "${#flags[@]}" == 0 ]]
    [[ ${help_flags} == false ]]
}


@test "Arg/Kwarg Handling - Flags Only" {
    # Import utils script.
    source ../utils.sh

    possible_flags=("-a" "--b" "-c" "--d")

    # Test with single dash.
    expected_result=("a" "c")
    handle_args_kwargs -a -c
    [[ "${#args[@]}" == 0 ]]
    [[ "${#kwargs[@]}" == 0 ]]
    [[ "${#flags[@]}" == 2 ]]
    [[ ${flags[@]} == ${expected_result[@]} ]]
    [[ ${help_flags} == false ]]

    # Ensure kwargs clear on new call.
    handle_args_kwargs
    [[ "${#flags[@]}" == 0 ]]

    # Test with double dash.
    expected_result=("b" "d")
    handle_args_kwargs --b --d
    [[ "${#args[@]}" == 0 ]]
    [[ "${#kwargs[@]}" == 0 ]]
    [[ "${#flags[@]}" == 2 ]]
    [[ ${flags[@]} == ${expected_result[@]} ]]
    [[ ${help_flags} == false ]]

    # Ensure kwargs clear on new call.
    handle_args_kwargs
    [[ "${#flags[@]}" == 0 ]]

    # Test with mixed dashes.
    expected_result=("a" "c" "b" "d")
    handle_args_kwargs -a -c --b --d
    [[ "${#args[@]}" == 0 ]]
    [[ "${#kwargs[@]}" == 0 ]]
    [[ "${#flags[@]}" == 4 ]]
    [[ ${flags[@]} == ${expected_result[@]} ]]
    [[ ${help_flags} == false ]]
}


@test "Arg/Kwarg Handling - Mixed Set" {
    # Import utils script.
    source ../utils.sh

    possible_flags=("-a" "--b")

    # Test two of each.
    expected_args=("arg1" "arg2")
    declare -A expected_kwargs=()
    expected_kwargs["aa"]="xx"
    expected_kwargs["bb"]="yy"
    expected_flags=("a" "b")
    handle_args_kwargs arg1 --aa xx -a arg2 --bb yy --b
    [[ "${#args[@]}" == 2 ]]
    [[ ${args[@]} == ${expected_args[@]} ]]
    [[ "${#kwargs[@]}" == 2 ]]
    [[ ${kwargs[@]} == ${expected_kwargs[@]} ]]
    [[ "${#flags[@]}" == 2 ]]
    [[ ${flags[@]} == ${expected_flags[@]} ]]
    [[ ${help_flags} == false ]]

    # Reset flag.
    help_flags=false
    [[ ${help_flags} == false ]]

    # Test two of each, different ordering.
    expected_args=("arg2" "arg1")
    expected_flags=("b" "a")
    handle_args_kwargs --b --bb yy arg2 -a --aa xx arg1
    [[ "${#args[@]}" == 2 ]]
    [[ ${args[@]} == ${expected_args[@]} ]]
    [[ "${#kwargs[@]}" == 2 ]]
    [[ ${kwargs[@]} == ${expected_kwargs[@]} ]]
    [[ "${#flags[@]}" == 2 ]]
    [[ ${flags[@]} == ${expected_flags[@]} ]]
    [[ ${help_flags} == false ]]
}
